/*
 *  main.c
 *  siemens-test
 *
 *  Simple test case for a Siemens cert which seems to blow
 *  up various bits & pieces. Brought back to the simplest case.
 *
 *   Copyright 2012 Dirk-Willem van Gulik, All Rights Reserved. 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
#include <stdio.h>
#include <assert.h>

#include <CoreFoundation/CoreFoundation.h>
#include <Security/Security.h>

const unsigned char buff[128*1024];

// Simple utility to suck in a DER cert from a file. Ignore, The issue
// is in the main.
//
static SecCertificateRef SecCertificateCreateFromFile(const char * fname) {    
    FILE * f;
    if (!(f= fopen(fname,"r"))) {
        perror("Failed to open");
        exit(2);
    }
    
    size_t len = fread((void*)buff,1,sizeof(buff),f);
    if (len == 0) {
        perror("Could not read");
        exit(3);
    }
    if (len == sizeof(buff)) {
        fprintf(stderr,"DER file '%s' too large.\n", fname);
        exit(4);
    }
    CFDataRef rawData = CFDataCreateWithBytesNoCopy(NULL, buff, len, NULL);
    assert(rawData);
    
    SecCertificateRef certificateRef = SecCertificateCreateWithData(NULL, rawData);
    assert(certificateRef);
    
    return certificateRef;
}

int main (int argc, const char * argv[])
{
    CFErrorRef error = NULL;
    
    if (argc > 2) {
        fprintf(stderr,"Syntax: %s [der-file-to-check]\n", argv[0]);
        exit(1);
    }        

    SecCertificateRef certificateRef = SecCertificateCreateFromFile(argc > 1 ? argv[0] : "siemens-fail.der");
            
//    const void *keys[] = { kSecOIDX509V1SubjectName, kSecOIDX509V1IssuerName };
//    const void *keys[] = { kSecOIDSerialNumber };
    const void *keys[] = { kSecOIDCommonName };
    
    CFArrayRef keySelection = CFArrayCreate(NULL, keys , sizeof(keys)/sizeof(keys[0]), &kCFTypeArrayCallBacks);
    
    // Caboom in next line.
    CFDictionaryRef vals = SecCertificateCopyValues(certificateRef, keySelection, &error);
    assert(vals);
    
    printf("Ok\n");
    return 0;
}

